"""
Custom widgets for the PCILeech TUI application.

This package contains custom widgets that extend the functionality
of the Textual UI framework for specific needs of the application.
"""

from .virtual_device_table import VirtualDeviceTable

__all__ = ["VirtualDeviceTable"]
